@@ -31,7 +31,7 @@ use Net::LDAP::Constant qw(LDAP_SUCCESS
 
 use constant CAN_IPV6 => eval { require IO::Socket::INET6 } ? 1 : 0;
 
-$VERSION 	= "0.47";
+$VERSION 	= "0.49";
 @ISA     	= qw(Tie::StdHash Net::LDAP::Extra);
 $LDAP_VERSION 	= 3;      # default LDAP protocol version
 
@@ -58,7 +58,7 @@ sub _options {
     $ret{substr($v,1)} = $ret{$v};
   }
 
-  $ret{control} = [ map { (ref($_) =~ /[^A-Z]/) ? $_->to_asn : $_ } 
+  $ret{control} = [ map { (ref($_) =~ /[^A-Z]/) ? $_->to_asn : $_ }
 		      ref($ret{control}) eq 'ARRAY'
 			? @{$ret{control}}
 			: $ret{control}
@@ -160,7 +160,7 @@ sub connect_ldap {
 		 ? $arg->{timeout}
 		 : 120
   ) or return undef;
-  
+
   $ldap->{net_ldap_host} = $host;
   $ldap->{net_ldap_port} = $port;
 }
@@ -271,7 +271,19 @@ sub connect_ldapi {
 		 : 120
   ) or return undef;
 
-  $ldap->{net_ldap_host} = 'localhost';
+  # try to get canonical host name [to allow start_tls on the connection]
+  require Socket;
+  if (Socket->can('getnameinfo') && Socket->can('getaddrinfo')) {
+    my @addrs;
+    my ($err,$host,$path) = Socket::getnameinfo($ldap->{net_ldap_socket}->peername, &Socket::AI_CANONNAME);
+
+    ($err, @addrs) = Socket::getaddrinfo($host, 0, { flags => &Socket::AI_CANONNAME } )
+      unless ($err);
+    map { $ldap->{net_ldap_host} = $_->{canonname}  if ($_->{canonname}) }  @addrs
+      unless ($err);
+  }
+
+  $ldap->{net_ldap_host} ||= 'localhost';
   $ldap->{net_ldap_peer} = $peer;
 }
 
@@ -341,9 +353,7 @@ sub unbind {
 }
 
 # convenience alias
-sub done {
-  goto &unbind;
-}
+*done = \&unbind;
 
 
 sub ldapbind {
@@ -461,7 +471,7 @@ sub bind {
 }
 
 
-my %scope = qw(base  0 one    1 single 1 sub    2 subtree 2);
+my %scope = qw(base  0 one    1 single 1 sub    2 subtree 2 children 3);
 my %deref = qw(never 0 search 1 find   2 always 3);
 
 sub search {
@@ -575,7 +585,7 @@ sub modify {
     while($j < @{$arg->{changes}}) {
       return _error($ldap, $mesg, LDAP_PARAM_ERROR,"Bad change type '" . $arg->{changes}[--$j] . "'")
        unless defined($opcode = $opcode{$arg->{changes}[$j++]});
-      
+
       $chg = $arg->{changes}[$j++];
       if (ref($chg)) {
 	my $i = 0;
@@ -843,7 +853,7 @@ sub _sendmesg {
   my $mid  = $mesg->mesg_id;
   my $sync = not $ldap->async;
 
-  unless ($mesg->done) { # may not have a responce
+  unless ($mesg->done) { # may not have a response
 
     $ldap->{net_ldap_mesg}->{$mid} = $mesg;
 
@@ -922,6 +932,7 @@ sub _drop_conn {
 
   if (my $msgs = delete $self->{net_ldap_mesg}) {
     foreach my $mesg (values %$msgs) {
+      next unless (defined $mesg);
       $mesg->set_error($err, $etxt);
     }
   }
